//#include <stdint.h>              
#include <string.h>
#include <absacc.h>
#include <intrins.h>
#include "..\inc\ch552_keil.h"

typedef unsigned short uint16_t; //org Keil does not have these
typedef unsigned char  uint8_t;

  // from Aarons board
  #define LED1   TXD      
  #define LED3   RXD 
  #define SW1    INT0

  #define LOW(x)  (*((unsigned char*)&(##x)+1))       // returns the low part
  #define HIGH(x) (*((unsigned char*)&(##x)+0))       // returns the high part
  #define CALL(addr)   (((void(*)(void))(uint8_t code *)addr)()) //call any address

  
/** a simple delay loop used for LED blinking */  
void delay(uint16_t Time)
{
  while(Time)
  {
     _nop_();
     _nop_();
     _nop_();
     _nop_();  
     Time--;    
  }    
}  

/** store one word into the codeflash */
uint8_t WriteCodeFlash(uint16_t Address, uint16_t Data)
{
   ROM_ADDR = Address;
   ROM_DATA = Data;
   if (ROM_CTRL & bROM_ADDR_OK)
   {
      ROM_CTRL = ROM_CMD_WRITE;
      return ROM_CTRL ^ bROM_ADDR_OK;
   }           
   return bROM_ADDR_OK;
}

uint16_t crc;

void crc16(uint8_t b)
{
   register uint8_t i;
   crc^=b;
   for(i=8;i;i--)
   {
     crc= (crc>>1)^((crc&1) ? 0xA001:0);
   }
}  


extern code unsigned char Image[0x07F0];

uint16_t BuildCrc(uint8_t *p)
{
   uint16_t i;
   crc=0; 
   for (i=0; i < sizeof(Image);i++)
   {
      crc16(*p);
      p++;
   }
   return crc;
}

void main (void)
{
   uint16_t  image_crc;
   uint16_t  loader_crc;
   uint16_t  addr; 
   uint16_t  value;
   uint16_t  i;   
   LED1 = 0;
   LED3 = 0; 
   P3_MOD_OC &= 0xFC;  // P3.0 and P3.1 PUSH PULL LED1 and LED3
// P3_DIR_PU  = 0xFF;  // this is the reset default
   if (SW1 == 0)       // Arrons board has a button there 
   {  //start the original bootloader
   	  CALL (0x3800); 
   }	 
 

   while(1)
   {
      delay(0xFFFF);
      LED1 =1;
      delay(0xFFFF);
      LED1 =0;
      if(SW1==0) break;
   }  
   LED1=0;
   crc=0; image_crc =BuildCrc(Image);
   crc=0; loader_crc=BuildCrc((uint8_t code *) 0x3000);
   if(image_crc==loader_crc)
   {
      for (i=0;i< 10;i++)
      {
         delay(0xFFFF);
         LED1 = 1;
         LED3 = 1; 
         delay(0xFFFF);
         LED1 = 0;
         LED3 = 0;
      }  
      CALL(0x3000);
      while(1);
   }
   // we need to flash
   else
   {  
      SAFE_MOD    = 0x55;
	    SAFE_MOD    = 0xAA;
	    GLOBAL_CFG |= bCODE_WE;
      SAFE_MOD    = 0;

      addr = 0x3000;
      for (i=0;i<sizeof(Image);i+=2)
      {        
         value=(Image[i+1] <<8) | Image[i];  
         WriteCodeFlash(addr,value);
         addr+=2;
      }
      SAFE_MOD    =  0x55;
	    SAFE_MOD    =  0xAA;
	    GLOBAL_CFG &= ~bCODE_WE;

      while(1)
      {
         delay(0xFFFF);
         LED3 = 1; 
         delay(0xFFFF);
         LED3 = 0;
      }
   }     
}